/*
 * InfoWindow.cpp
 *
 * Written by Matt Pauker
 * 
 * Last modified: 9/15/96
 */

#ifndef INFO_WINDOW_H
#include "InfoWindow.h"
#endif

#ifndef CHANGE_APPLICATION_H
#include "ChangeApplication.h"
#endif

#ifndef CHANGE_CONSTNATS_H
#include "ChangeConstants.h"
#endif

#ifndef CHANGE_UTILS_H
#include "ChangeUtils.h"
#endif

#ifndef MESSAGE_DROPPED_FILTER_H
#include "MessageDroppedFilter.h"
#endif

#ifndef SIGNATURE_VIEW_H
#include "SigView.h"
#endif

InfoWindow::InfoWindow(long topX, long topY) : 
	BWindow(BRect(topX, topY, topX + 190, topY + 90), "", B_TITLED_WINDOW, B_NOT_RESIZABLE|B_NOT_ZOOMABLE)
{
	// Set up menus
	
	BMenu *myMenu;
	BMenuItem *myMenuItem;
	BRect menuBarRect;
	
	menuBarRect = Bounds();
	menuBarRect.bottom = menuBarRect.top + 14.0;
	
	menuBar = new BMenuBar(menuBarRect, "MenuBar", B_FOLLOW_LEFT_RIGHT|B_FOLLOW_TOP, B_ITEMS_IN_ROW, FALSE);
	
	// File menu
	myMenu = new BMenu("File", B_ITEMS_IN_COLUMN);
	menuBar->AddItem(myMenu);
	myMenuItem = new BMenuItem("Save", new BMessage(SAVE_INFO), 'S', NULL);
	myMenuItem->SetEnabled(FALSE);
	myMenu->AddItem(myMenuItem);
	myMenuItem = new BMenuItem("Revert", new BMessage(REVERT_INFO), NULL, NULL);
	myMenuItem->SetEnabled(FALSE);
	myMenu->AddItem(myMenuItem);
	myMenuItem = new BMenuItem("Close", new BMessage(B_QUIT_REQUESTED), 'W', NULL);
	myMenu->AddItem(myMenuItem);
	
	// Edit menu
	myMenu = new BMenu("Edit", B_ITEMS_IN_COLUMN);
	menuBar->AddItem(myMenu);
	myMenuItem = new BMenuItem("Cut", new BMessage(B_CUT), 'X');
	myMenuItem->SetTarget(this, TRUE);
	myMenu->AddItem(myMenuItem);
	myMenuItem = new BMenuItem("Copy", new BMessage(B_COPY), 'C');
	myMenuItem->SetTarget(this, TRUE);
	myMenu->AddItem(myMenuItem);
	myMenuItem = new BMenuItem("Paste", new BMessage(B_PASTE), 'V');
	myMenuItem->SetTarget(this, TRUE);
	myMenu->AddItem(myMenuItem);
	myMenuItem = new BMenuItem("Delete", new BMessage(B_DELETE), NULL);
	myMenuItem->SetTarget(this, TRUE);
	myMenu->AddItem(myMenuItem);
	myMenuItem = new BMenuItem("Select All", new BMessage(SELECT_ALL), 'A');
	myMenuItem->SetTarget(this, TRUE);
	myMenu->AddItem(myMenuItem);
	
	AddChild(menuBar);
	SetKeyMenuBar(menuBar);

	filesMenu = NULL;
	
	// Type text control

	typeView = new SignatureView(BRect(10, 30, 85, 45), "type", "Type:");
	AddChild(typeView);
		
	// App text control

	appView = new SignatureView(BRect(10, 60, 85, 75), "app", "App:");
	AddChild(appView);
	
	// Setup 'Choose...' buttons
	
	BButton *button;
	BRect buttonFrame;
	
	buttonFrame.Set(105, 25, 175, 50);
	button = new BButton(buttonFrame, "ChooseType", "Choose...", new BMessage(CHOOSE_TYPE),
												B_FOLLOW_LEFT|B_FOLLOW_TOP, B_WILL_DRAW);
	AddChild(button);
	
	buttonFrame.Set(105, 55, 175, 80);
	button = new BButton(buttonFrame, "ChooseType", "Choose...", new BMessage(CHOOSE_APP),
												B_FOLLOW_LEFT|B_FOLLOW_TOP, B_WILL_DRAW);
	AddChild(button);
	

	refList = new LockableList();			// Initialize list of refs

	myFilter = new MessageDroppedFilter(this);
	Lock();
	AddCommonFilter(myFilter);
	Unlock();
}

InfoWindow::~InfoWindow()
{
	long x, numItems;
	record_ref *item;
	
	refList->Lock();
	numItems = refList->CountItems();
	for (x = 0; x < numItems; x++)
	{
		item = (record_ref *)refList->RemoveItem((long)0);
		delete item;
	}
	refList->Unlock();
	
	delete refList;
	
	// Remove the filter and delete it
	
	RemoveFilter(myFilter);
	delete myFilter;
}

void InfoWindow::AddRef(record_ref ref)
{
	record_ref *newRef;
	long numItems;
	
	newRef = new record_ref;
	*newRef = ref;
	refList->AddItem(newRef);
	
	numItems = refList->CountItems();
	
	if (numItems == 1)
	{
		// Set the title of the window to the name of the file

		BFile file;
		char *name = new char[B_FILE_NAME_LENGTH+1];
	
		file.SetRef(ref);
		file.GetName(name);
		SetTitle(name);
		delete[] name;
	
		// Get file's type and app, save them, and put them in the text boxes
	
		ulong type, app;
	
		file.GetTypeAndApp(&type, &app);
		savedType = type;
		savedApp = app;
		
		Lock();
		typeView->SetSig(type);
		appView->SetSig(app);
		Unlock();
		
		// Make the type view the focus and select all its text
	
		Lock();
		typeView->MakeFocus();
		typeView->SelectAll();
		Unlock();		
	}
	else if (numItems >= 2)
	{
		if (numItems == 2)
		// It's now a batch change, so adjust the window
		{
			SetTitle("Batch Change");
			
			// Add the Files menu
			BMenuItem *myMenuItem;
	
			filesMenu = new BMenu("Files", B_ITEMS_IN_COLUMN);
			menuBar->AddItem(filesMenu);
			filesMenu->AddSeparatorItem();
			myMenuItem = new BMenuItem("All", new BMessage(ALL_ON), NULL, NULL);
			filesMenu->AddItem(myMenuItem);
			myMenuItem = new BMenuItem("None", new BMessage(ALL_OFF), NULL, NULL);
			filesMenu->AddItem(myMenuItem);
	
			long x, numItems;
			char *name = new char[B_FILE_NAME_LENGTH+1];
			BFile file;
			record_ref ref;
	
			refList->Lock();
			
			// Add the files to the Files menu
			for (x = 0; x <= 1; x++)
			{
				ref = *((record_ref *)refList->ItemAt(x));
				file.SetRef(ref);
				file.GetName(name);
		
				myMenuItem = new BMenuItem(name, new BMessage(FILE_SELECTED), NULL, NULL);
				myMenuItem->SetMarked(TRUE);
				filesMenu->AddItem(myMenuItem, x);
			}
			refList->Unlock();
			
			delete[] name;
		
			SetChanged(TRUE);
		}
		else
		// Already a Batch Change, so just need to add the ref to the Files menu
		{
			BMenuItem *newItem;
			BFile file;
			char *name = new char[B_FILE_NAME_LENGTH+1];

			file.SetRef(ref);
			file.GetName(name);
			
			// Create a new menu item and insert it just before the separator
			newItem = new BMenuItem(name, new BMessage(FILE_SELECTED), NULL, NULL);
			newItem->SetMarked(TRUE);
			filesMenu->AddItem(newItem, (filesMenu->CountItems() - 3));
			
			delete[] name;
			
			SetChanged(TRUE);
		}
	}
}

void InfoWindow::AddRefsInDir(record_ref dirRef, bool doRecursively)
// Adds all the files in the directory represented by dirRef to the file list
// If doRecursively is TRUE, AddRefsInDir recurses down the directory tree, adding files
{
	long numFiles, x;
	BFile file;
	BDirectory dir;
	
	dir.SetRef(dirRef);
	numFiles = dir.CountFiles();

	// Add all the files in the dir
	for (x = 0; x < numFiles; x++)
	{
		dir.GetFile(x, &file);
		AddRef(file.Record()->Ref());
	}
	
	if (doRecursively)
	{
		long numDirs;
		BDirectory subDir;

		numDirs = dir.CountDirectories();
		 
		// Call the function for all the subdirectories
		for (x = 0; x < numDirs; x++)
		{
			dir.GetDirectory(x, &subDir);
			AddRefsInDir(subDir.Record()->Ref(), doRecursively);
		}
	}
}

void InfoWindow::NextFocus()
// Changes the focus from the current view to the other
{
	SignatureView *nextView;
	
	if ((SignatureView *)CurrentFocus() == typeView)
		nextView = appView;
	else
		nextView = typeView;
	nextView->MakeFocus();
	nextView->SelectAll();
}

void InfoWindow::MessageReceived(BMessage *message)
{
	BMessage *msg;
	BFile file;
	ulong type, app;
	record_ref ref;
	ChangeApplication *theApp;
	long numItems, x;
	
	theApp = (ChangeApplication *)be_app;
	
	switch (message->what)
	{
		case SAVE_INFO:		// User chose 'Save'
			DoSave();
		break;
		
		case REVERT_INFO:	// User chose 'Revert'
			typeView->SetSig(savedType);
			appView->SetSig(savedApp);
		break;
		
		case CHOOSE_TYPE:		// User clicked 'Choose...' button next to type field			
			msg = new BMessage(TYPE_CHOSEN);
			msg->AddObject("window", this);
			
			theApp->ShowFilePanel("ChangeType : Choose Type...", "Choose", msg);
		break;
		
		case CHOOSE_APP:		// User clicked 'Choose...' button next to app field
			msg = new BMessage(APP_CHOSEN);
			msg->AddObject("window", this);
			
			theApp->ShowFilePanel("ChangeType : Choose App...", "Choose", msg);
		break;
		
		case TYPE_CHOSEN:		// Got file to use for type field			
			ref = message->FindRef("refs");
			if (does_ref_conform(ref, "File"))
			{
				file.SetRef(ref);
				file.GetTypeAndApp(&type, &app);
				
				typeView->SetSig(type);
			}
		break;
		
		case APP_CHOSEN:		// Got file to use for app field
			ref = message->FindRef("refs");
			if (does_ref_conform(ref, "File"))
			{
				file.SetRef(ref);
				file.GetTypeAndApp(&type, &app);
				
				appView->SetSig(app);
			}
		break;
		
		case FILE_SELECTED:		// User chose an item in the Files menu, so mark/unmark
			BMenuItem *theItem;
			
			theItem = (BMenuItem *)message->FindObject("source");
			if (theItem->IsMarked())
				theItem->SetMarked(FALSE);
			else
				theItem->SetMarked(TRUE);
		break;
		
		case ALL_ON:			// User chose 'All' from the Files menu, so mark all items
			numItems = refList->CountItems();
			
			for (x = 0; x < numItems; x++)
				filesMenu->ItemAt(x)->SetMarked(TRUE);
		break;
		
		case ALL_OFF:			// User chose 'None' from the Files menu, so unmark all
			numItems = refList->CountItems();
			
			for (x = 0; x < numItems; x++)
				filesMenu->ItemAt(x)->SetMarked(FALSE);
		break;	
		
		case TEXT_CHANGED:
			SetChanged(TRUE);
		break;
		
		
		default:
			BWindow::MessageReceived(message);
		break;
	}
}


void InfoWindow::SaveFile(record_ref ref)
{
	BFile file;
	
	// Save the type and app to the file represented by ref
	file.SetRef(ref);
	file.SetTypeAndApp(savedType, savedApp);
}

bool InfoWindow::DoSave()
// Tries to save type & app signatures - They must be either 0 or 4 characters long
// Returns TRUE if successful, FALSE if not
{
	int typeLen, appLen;
	BAlert *errorAlert;
	
	typeLen = typeView->TextLength();
	appLen  = appView->TextLength();
	
	// Make sure the signatures are of valid lengths
	if ((typeLen != 0) && (typeLen != 4))
	{
		errorAlert = new BAlert("", "Type signature must be either 0 or 4 characters long.", "OK",
														NULL, NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT);
		errorAlert->Go();
		return FALSE;
	}
	else if ((appLen != 0) && (appLen != 4))
	{
		errorAlert = new BAlert("", "Application signature must be either 0 or 4 characters long.",
														"OK", NULL, NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT);
		errorAlert->Go();
		return FALSE;
	}
	else
	{
		savedType = typeView->GetSig();
		savedApp  = appView->GetSig();
		
		long x, numItems;
		
		// Save the type and app for each file in refList
		numItems = refList->CountItems();
		for (x = 0; x < numItems; x++)
			if ((filesMenu == NULL) || (filesMenu->ItemAt(x)->IsMarked()))
				SaveFile(*(record_ref *)refList->ItemAt(x));
				
	
		SetChanged(FALSE);		// Disable 'Revert' and similar things
		
		return TRUE;
	}
}


bool InfoWindow::QuitRequested()
{
	ulong currentType, currentApp;
	long buttonPressed;
	bool doClose;
	
	currentType = typeView->GetSig();
	currentApp  = appView->GetSig();
	
	doClose = TRUE;
	
	// Check to see if the user has adjusted the type or app since the last save/open
	if ((currentType != savedType) || (currentApp != savedApp))
	{
		BAlert *myAlert;
		char text[128];
	
		// Put up an alert to make sure	
		sprintf(text, "Save changes to \"%s\" before closing?\n", Title());
		
		myAlert = new BAlert("", text, "Don't Save", "Cancel", "Save", B_WIDTH_AS_USUAL,
													B_WARNING_ALERT);
		myAlert->SetShortcut(1, B_ESCAPE);
		myAlert->SetShortcut(2, B_ENTER);
		
		buttonPressed = myAlert->Go();
		
		if (buttonPressed == 0)				// User pressed 'Don't Save'
		{
			doClose = TRUE;
		}
		else if (buttonPressed == 1)	// User pressed 'Cancel'
		{
			doClose = FALSE;
		}
		else if (buttonPressed == 2)	// User pressed 'Save'
		{
			doClose = DoSave();	
		}
	}
	if (doClose)	// We need to close the window
	{
		((ChangeApplication *)be_app)->WindowClosed(this);		// Let the app know we're closing
		if (be_app->CountWindows() == 1)					// If we're closing the only window open, quit
			be_app->PostMessage(B_QUIT_REQUESTED);
	}
	return doClose;
}

bool InfoWindow::HasRef(record_ref theRef)
// Returns whether or not theRef is in refList
{
	long numItems, x;
	bool found;
	record_ref ref;
	
	found = FALSE;
	
	refList->Lock();
	numItems = refList->CountItems();
	for (x = 0 ; x < numItems; x++)
	{
		ref = *((record_ref *)refList->ItemAt(x));
		if (ref == theRef)
		{
			found = TRUE;
		}
	}
	refList->Unlock();

	return found;
}

void InfoWindow::Mark(record_ref ref)
// Marks the 'Files' menu item that refers to the file represented by ref (Assuming it's a Batch Change)
{
	if (filesMenu != NULL)
	{
		BFile file;
		char *name = new char[B_FILE_NAME_LENGTH+1];
		
		// Get the name of the file
		file.SetRef(ref);
		file.GetName(name);
		
		BMenuItem *theItem;
		
		// Find the item in the menu and mark it
		theItem = filesMenu->FindItem(name);
		theItem->SetMarked(TRUE);
	
		delete[] name;
	}
}

void InfoWindow::SetChanged(bool flag)
// Adjusts the enabled state of various menu items depending on whether the window has been changed
// since the last save
{
	BMenu *fileMenu;
	BMenuItem *menuItem;
	
	fileMenu = menuBar->FindItem("File")->Submenu();
	menuItem = fileMenu->FindItem("Save");
	menuItem->SetEnabled(flag);
	menuItem = fileMenu->FindItem("Revert");
	menuItem->SetEnabled(flag);
}

void InfoWindow::ResetSigs()
// Resets the type and app signatures
{
	Lock();
	appView->SetSig(0);
	typeView->SetSig(0);
	Unlock();
	savedApp = 0;
	savedType = 0;
}
