/*
 * MessageDroppedFilter.cpp
 *
 * Written by Matt Pauker
 *
 * Last modified: 9/15/96
 */

#ifndef MESSAGE_DROPPED_FILTER_H
#include "MessageDroppedFilter.h"
#endif

#ifndef CHANGE_UTILS_H
#include "ChangeUtils.h"
#endif

#ifndef INFO_WINDOW_H
#include "InfoWindow.h"
#endif

#ifndef SIG_VIEW_H
#include "SigView.h"
#endif

MessageDroppedFilter::MessageDroppedFilter(InfoWindow *theWindow) :
	BMessageFilter(B_DROPPED_DELIVERY, B_REMOTE_SOURCE)
{
	ourWindow = theWindow;
	
	typeView = (SignatureView *)ourWindow->FindView("type");
	appView = (SignatureView *)ourWindow->FindView("app");
}

filter_result MessageDroppedFilter::Filter(BMessage *message, BHandler **target)
{
	record_ref ref;
	long numRefs, x;
	ulong type;

	message->GetInfo("refs", &type, &numRefs);
	for (x = 0; x < numRefs; x++)
	{
		// Get the ref from the message and retrieve the type and app
		ref = message->FindRef("refs", x);
		if (does_ref_conform(ref, "File"))
		{	
			// If the user was holding down the control key when they dragged, add the file's ref
			if (IsKeyDown(B_CONTROL_KEY))
				if (!ourWindow->HasRef(ref))
					ourWindow->AddRef(ref);
				else
					ourWindow->Mark(ref);
			// Otherwise, adjust the type/app signatures
			else
			{
				BFile file;
				ulong type, app;
	
				file.SetRef(ref);
				file.GetTypeAndApp(&type, &app);
			
				// If the user dragged the file onto the type or app box, set the signature for that box
				// If the user dragged the file anywhere else on the window, set them both
				if ((*target == typeView) || (((BView *)(*target))->Parent() == typeView))
				{
					typeView->SetSig(type);
					typeView->MakeFocus();
				}
				else if ((*target == appView) || (((BView *)(*target))->Parent() == appView))
				{
					appView->SetSig(app);
					appView->MakeFocus();
				}
				else
				{
					typeView->SetSig(type);
					appView->SetSig(app);
				}
				
				ourWindow->SetChanged(TRUE);
			}
		}
	}	
	return B_SKIP_MESSAGE;
}